unit TBtnClose;

//* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *//
//* TBtnClose. Delphi 3, 4
//*
//* This is like regular TBitBtn. I've just added internal timer that counts
//* down and closes the Parent form when counter reaches Zero. The progress of
//* the counting is indicated on the button.
//* It's sutable for form which must be closed automatically after X seconds.
//* The Parent form will be closed aloso by pressing in the button in every
//* moment
//*
//* Writen by Paul Krestol.
//* For contacts e-mail me to : paul@mediasonic.co.il
//* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *//

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons;

type
  TBtnClose = class( TBitBtn )
  private
    FAutoClose : boolean;
    FCounting : boolean;
    FEnabled : boolean;
    FRemaining : longint;
    FOnClick: TNotifyEvent;
    FTimeOut : cardinal;
    FWindowHandle: HWND;
    procedure CloseForm;
    procedure MyOnClick( Sender: TObject );
    procedure SetCaption;
    procedure SetCounting( Value : boolean );
    procedure StartTimer;
    procedure StopTimer;
    procedure TimerProc;
    procedure WndProc(var Msg: TMessage);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property AutoClose : boolean read FAutoClose write FAutoClose;
    property Counting : boolean read FCounting write SetCounting;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property TimeOut : cardinal read FTimeOut write FTimeOut;
  end;

procedure Register;
implementation
{$R *.RES}

(******************************************************************************)
constructor TBtnClose.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  FWindowHandle := AllocateHWnd( WndProc );
  FTimeOut := 10;
  FAutoClose := False;
  FCounting := False;
  Width := 110;
  inherited OnClick := MyOnClick; //** override original OnClick method **//
end;

(******************************************************************************)
destructor TBtnClose.Destroy;
begin
  FEnabled := False;
  StopTimer;
  DeallocateHWnd( FWindowHandle );
  inherited Destroy;
end;

(******************************************************************************)
procedure TBtnClose.MyOnClick( Sender: TObject );
begin
  if Assigned( FOnClick ) then FOnClick( Sender );
  CloseForm; //** Always close form on click **//
end;

(******************************************************************************)
procedure TBtnClose.WndProc(var Msg: TMessage);
begin
  with Msg do begin
    if Msg = WM_TIMER then
      try
        TimerProc;
      except
        Application.HandleException(Self);
      end
    else
      Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
  end;
end;

(******************************************************************************)
procedure TBtnClose.SetCounting( Value : boolean );
begin
  if FCounting = Value then Exit;
  FCounting := Value;
  case FCounting of
    True : begin
      StartTimer;
    end;
    False : begin
      StopTimer;
    end
  end;
end;

(******************************************************************************)
procedure TBtnClose.SetCaption;
begin
  if (csDesigning in ComponentState) then begin
    Caption := 'Closing in ' + IntToStr( FTimeOut ) + ' Secs';
    Exit;
  end;
  case FCounting of
    True : Caption := 'Closing in ' + IntToStr( FRemaining ) + ' Secs';
    False : Caption := 'Close';
  end;
end;

(******************************************************************************)
procedure TBtnClose.CloseForm;
var
  Component : TWinControl;
begin
  Component := Self;
  while True do begin
    if Component.Parent is TForm then begin
      (Component.Parent as TForm).Close;
      Break;
    end;
    if not Assigned( Component.Parent ) then Break;
    Component := Component.Parent;
  end;
end;

(******************************************************************************)
procedure TBtnClose.TimerProc;
begin
  SetCaption;
  if FRemaining = 0 then begin
    StopTimer;
    if FAutoClose then begin
      CloseForm;
    end;
  end;
  Dec( FRemaining );
end;

(******************************************************************************)
procedure TBtnClose.StartTimer;
begin
  if (csDesigning in ComponentState) then Exit;
  KillTimer( FWindowHandle, 1 );
  if FCounting then
    if SetTimer(FWindowHandle, 1, 1000, nil) = 0 then
      raise EOutOfResources.Create('System is low resources. No timers');
  FRemaining := FTimeOut;
  TimerProc;
end;

(******************************************************************************)
procedure TBtnClose.StopTimer;
begin
  if (csDesigning in ComponentState) then Exit;
  KillTimer( FWindowHandle, 1 );
  FCounting := False;
  SetCaption;
end;

(******************************************************************************)
(******************************************************************************)
(******************************************************************************)
procedure Register;
begin
  RegisterComponents('Plus', [TBtnClose]);
end;

end.
